/// <reference types="express" />
import { Server } from "./server";
import { Express, Request, Response, NextFunction } from "./express";
import { IContract, ROUTE, IMiddlewareOptions, MIDDLEWARE, IRouteOptions } from "./contract";
import { CODES } from "tools-box/fetch/codes";
import { Logger } from './logger';
import { IServiceEvents } from "./interface";
export interface IServiceOptions {
    addRootName?: boolean;
    authService?: string;
    logService?: string;
    exitOnError?: boolean;
    bodyQuota?: number;
    hostname?: string;
    port?: number;
    protocol?: string;
}
export { ROUTE, MIDDLEWARE, IServiceEvents, IContract, CODES, Request, Response, NextFunction, Express };
export declare class Service {
    private _env;
    private _negateRootName;
    readonly _contract: IContract;
    readonly _routes: {
        [key: string]: IRouteOptions;
    };
    readonly _middlewares: {
        [key: string]: IMiddlewareOptions;
    };
    readonly bodyQuota: number;
    readonly authService: string;
    protected server: Server;
    logger: Logger;
    constructor(options?: IServiceOptions);
    readonly env: string;
    readonly contract: IContract;
    error(message: string, original?: any, code?: string): {
        message: string;
        original: any;
        code: string;
    };
    run(): void;
    protected authorize(route: string, req?: Request): Promise<any> | ((req: Request, res?: Response, next?: NextFunction) => Promise<void | import("express-serve-static-core").Response>);
}
