"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const server_1 = require("./server");
const express_1 = require("./express");
const extend_1 = require("tools-box/object/extend");
const contract_1 = require("./contract");
exports.ROUTE = contract_1.ROUTE;
exports.MIDDLEWARE = contract_1.MIDDLEWARE;
const codes_1 = require("tools-box/fetch/codes");
exports.CODES = codes_1.CODES;
const logger_1 = require("./logger");
const fetch_1 = require("tools-box/fetch");
function _authorize(authService, req, serviceName, route) {
    return __awaiter(this, void 0, void 0, function* () {
        let auth = req.headers.authorization ? req.headers.authorization.split(' ')[1] : null;
        let body = req.body;
        let params = req.params;
        let query = req.query;
        if (!auth)
            return null;
        req.auth = auth;
        let res = yield fetch_1.default({
            url: authService,
            headers: { Authorization: `basic ${auth}` },
            method: 'POST',
            data: {
                service: serviceName,
                route,
                payload: { body, params, query }
            }
        });
        return res.data;
    });
}
class Service {
    constructor(options = {}) {
        this._env = process.env.NODE_ENV || (process.env.NODE_ENV = 'dev');
        this._negateRootName = !options.addRootName;
        this.authService = options.authService;
        this.logger = new logger_1.Logger(this.constructor.name, this._env, options.logService, !!options.exitOnError);
        this.bodyQuota = options.bodyQuota || this.bodyQuota;
        this.server = new server_1.Server({
            hostname: options.hostname || 'localhost',
            port: options.port || 7777,
            protocol: options.protocol || 'http'
        }, this.logger);
        this._contract = {
            name: this.constructor.name,
            path: `/${this.constructor.name.toLowerCase()}`,
            hostname: this.server.hostname || 'localhost',
            port: this.server.port,
            protocol: this.server.protocol || 'http:'
        };
    }
    get env() {
        return this._env;
    }
    get contract() {
        return extend_1.extend({}, this._contract, true);
    }
    error(message, original, code) {
        this.logger.error(message);
        console.trace(message);
        if (original)
            this.logger.error(original.message, original);
        return { message, original: (original ? original.message : original) || null, code };
    }
    run() {
        express_1.initExpress(this.server.app, this, this._negateRootName);
        this.server.run(() => {
            if (this.hasOwnProperty('onRunning'))
                this.onRunning();
        });
    }
    authorize(route, req) {
        if (req)
            return _authorize(this.authService, req, this._contract.name, route);
        return (req, res, next) => __awaiter(this, void 0, void 0, function* () {
            if (!this.authService || !this._routes[route] || !this._routes[route].authorization.enabled)
                return next();
            let origins = this._routes[route].authorization.skipOrigins || [];
            for (let i = 0; i < origins.length; i++) {
                const origin = origins[i];
                if (req.headers.host.indexOf(origin) > -1)
                    return next();
            }
            try {
                let user = yield _authorize(this.authService, req, this._contract.name, route);
                if (!user)
                    return res.status(codes_1.CODES.TOKEN_REQUIRED).json({ message: 'missing_token' });
                req.user = user;
                next();
            }
            catch (e) {
                if (e.statusCode && e.error)
                    return res.status(e.statusCode).json(e.error);
                res.status(codes_1.CODES.UNKNOWN_ERROR).json(this.error('error_authorizing_request!', e));
            }
        });
    }
}
exports.Service = Service;
//# sourceMappingURL=index.js.map