"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const winston = require("winston");
const Transport = require("winston-transport");
const path_1 = require("path");
const fs = require("fs");
const WDRF = require("winston-daily-rotate-file");
const index_1 = require("tools-box/fetch/index");
const rootLogDir = path_1.join(process.cwd(), 'logs');
const errorsLogsDir = path_1.join(rootLogDir, 'errors');
const excpetionsLogsDir = path_1.join(rootLogDir, 'excpetions');
const myFormat = winston.format.printf(info => {
    return `${info.level}: ${info.message} - ${info.timestamp}`;
});
class LogService extends Transport {
    constructor(serviceName, logService) {
        super();
        this.serviceName = serviceName;
        this.logService = logService;
    }
    log(info, next) {
        index_1.default({
            url: this.logService,
            method: 'POST',
            data: { service: this.serviceName, info },
            wait: false
        });
        next();
    }
}
class Logger {
    constructor(serviceName, env = 'dev', logService, exitOnError = true) {
        this.serviceName = serviceName;
        this.logService = logService;
        this.env = env;
        this.exitOnError = exitOnError;
        if (env === 'prod')
            this.initProd();
        else
            this.initDev();
    }
    initProd() {
        if (!this.logService) {
            fs.existsSync(rootLogDir) || fs.mkdirSync(rootLogDir);
            fs.existsSync(errorsLogsDir) || fs.mkdirSync(errorsLogsDir);
            fs.existsSync(excpetionsLogsDir) || fs.mkdirSync(excpetionsLogsDir);
        }
        winston.configure({
            exitOnError: this.exitOnError,
            format: winston.format.combine(winston.format.timestamp(), myFormat),
            transports: [
                this.logService ? new LogService(this.serviceName, this.logService) : new WDRF({ filename: path_1.join(errorsLogsDir, '%DATE%.log'), level: 'error' })
            ],
            exceptionHandlers: [
                this.logService ? new LogService(this.serviceName, this.logService) : new WDRF({ filename: path_1.join(excpetionsLogsDir, '%DATE%.log') })
            ]
        });
    }
    initDev() {
        winston.configure({
            exitOnError: true,
            format: winston.format.combine(winston.format.colorize({ colors: { info: 'blue', error: 'red', warn: 'magenta' } }), winston.format.simple()),
            transports: [new winston.transports.Console()],
            exceptionHandlers: [new winston.transports.Console()]
        });
    }
    info(message, ...meta) {
        if (this.env === 'prod')
            return;
        winston.info(message, meta);
    }
    warn(message, ...meta) {
        if (this.env === 'prod')
            return;
        winston.warn(message, meta);
    }
    error(message, ...meta) {
        winston.error(message, meta);
    }
}
exports.Logger = Logger;
//# sourceMappingURL=logger.js.map